;MTS15CTL,ASM.ASM     G4JNt  July 2009

;Control Verticom MTS1500 Synths
;Full control via RS232 interface using port change interrupt
;  Compatible with PIC controller from G4FRE

;  Serial Command  protocol     Rxxxx[cr]    Input new R data, no update
;                               Nxxxxx[cr]   Input new N data, no update
;                               Fxxxxx[cr]   Input new F data, no update
;                               U [cr]       Update chip with new data, don't store
;                               W [cr]       Update chip and write to EE

    PROCESSOR 12F629
    INCLUDE "P12F629.INC"
    errorlevel 1

;   ===============================================================

    #define     SerIn       GPIO, 0     ;Also PGD,  Port change enabled
    #define     SerOut      GPIO, 1     ;Also PGC
    #define     LMXStrb     GPIO, 2	;I/O LInes compatible with MTS1500.ASM
    #define     LMXData     GPIO, 4	;     by G4FRE
    #define     LMXClock    GPIO, 5

    #define DEBUG   Flags, 0      ;1 = send synth data on RS232

 RS232POL   =   1           ; 1 = RS232 levels,  0 = Logic/FTDI Interface
 BAUDCONST  =   d'138'      ;Half baud delay, 1200 baud

    

  __config  0x3F94      ;INTOSC, I/O on GP4/5
  ;CPD Off, CP Off, Boden Off, MCLRE = I/O, PWRTE Off, WDT Off,
  __idlocs  0x8080

    Rvalue = d'63'
    Pvalue = d'108'
    Svalue = d'28'
    Gvalue = d'4'
    Cvalue = d'488'

  org   0x2100
Rdat    de  HIGH(Rvalue), LOW(Rvalue)     ;R data  (14 bits in total)
Pdat    de  HIGH(Pvalue), LOW(Pvalue)     ;P data  (11 bits)
Sdat    de  Svalue              ;        ( 5 bits)
Cdat    de  HIGH(Cvalue), LOW(Cvalue)     ;C data  (10 bits)
Gdat    de  Gvalue              ;        ( 6 bits)


  org   0
    nop
    clrw
    movwf   INTCON          ;disable interrupts
    goto    StartUp         ;jump to main code
;================================
  org 4     ;Interrupt routine, every time RotI goes low to high
    movwf   Wreg            ;Save W
    swapf   STATUS , W      ; nb.  movwf does not affect any STATUS bits
    movwf   StsReg          ;Stave STATUS, without affecting it

    btfss   INTCON , GPIF   ;Test for port B change
    goto    CarryOnInt      ;
                            ;
    bcf     INTCON , GPIE   ;
    call    Assemble232     ;MUSTN'T Waste ANY TIME UP TO HERE !!!!

    sublw   0x0D            ;Check for [cr] terminating any command,
    btfsc   STATUS, Z       ;  otherwise chars are pushed onto stack
    call    CheckSerCmd     ;Decode the command and do any actions

    bcf     INTCON , GPIF   ;Clear the flag
    bsf     INTCON , GPIE   ;Re-enable the interrupt
;...............
CarryOnInt
    swapf   StsReg , W      ;Restore STATUS
    movwf   STATUS
    swapf   Wreg            ;Restore W
    swapf   Wreg , W        ; nb.  movf affects Z bit
    retfie
;=================================
StartUp 
    movlw   7
    movwf   CMCON

	bsf     STATUS,RP0      ;ram page 1

;    clrf    ANSEL           ;Sets all pins as digital  only on 12F675

    movlw   b'00000001'     ;GP0 is the serial input, all others out
    movwf   TRISIO          ;
    movwf   IOC             ;Port change interrupt on GP0

    movlw   b'00000000'     ;Pull-ups enabled
	movwf	OPTION_REG

    call    0x3FF           ;Get factory stored OSCCAL value
    movwf   OSCCAL

    bcf     STATUS, RP0

    bcf     LMXStrb
    clrf    Flags

  if RS232POL == 1
    bcf   SerOut      ;RS232 idle state (note RS232 polarity !)
  else
    bsf   SerOut
  endif

    call    LongDelay       ;Give it some time to settle
    call    IntroMessage

    movlw   LOW(Rdat)
    call    GetEE_12F
    movwf   R1
    
    movlw   LOW(Rdat) + 1
    call    GetEE_12F
    movwf   R0
    
    movlw   LOW(Pdat)
    call    GetEE_12F
    movwf   P1
    
    movlw   LOW(Pdat + 1)
    call    GetEE_12F
    movwf   P0

    movlw   LOW(Sdat)
    call    GetEE_12F
    movwf   S0

    movlw   LOW(Cdat)
    call    GetEE_12F
    movwf   C1
    
    movlw   LOW(Cdat) + 1
    call    GetEE_12F
    movwf   C0

    movlw   LOW(Gdat)
    call    GetEE_12F
    movwf   G0

    call    SendData                        
    call    SendBack

    movlw   b'10001000'      ;Port Change interrupt
    movwf   INTCON

MainLoop        ;
    goto    MainLoop

;------------------------------------
SendData                ;Now Send complete R  N and F register set        
    movf    R0, W
    movwf   D4
    movf    R1, W
    movwf   D5

    rlf     D4
    rlf     D5
    bsf     D5, 7       ;Align and set MSB & LSB  as per 'FRE's note
    bsf     D4, 0

    clrf    D0
    clrf    D1      
    clrf    D2
    clrf    D3


    btfss   DEBUG
    goto    NoDebug1
    movlw   "C"
    call    SendByte
    movlw   "W"
    call    SendByte
    movlw   "1"
    call    SendByte
    movlw   " "
    call    SendByte
    movlw   d'40'
    call    DebugSendBack
NoDebug1

    movlw   d'40'       ;40 bits in R word
    call    ClockItIn

Send_N_Only
    movf    P0, W       ;P has 11 bits
    movwf   D4
    movf    P1, W
    movwf   D5
                                    ;Bring in 5 bit S, place so 3MSBs
    rlf     S0, W                   ;  end up in D4 LSBs with 2 zeros
    movwf   D3      ;   D5       D4       D3
                    ;oooooPPP PPPPPPPP ooSSSSSo

    rlf     D3
    rlf     D4
    rlf     D5        
    rlf     D3
    rlf     D4
    rlf     D5
    rlf     D3
    rlf     D4
    rlf     D5
    rlf     D3
    rlf     D4
    rlf     D5
    rlf     D3
    rlf     D4          ;5 shifts
    rlf     D5          ;Align MS (Bit 10) of P at LH Side

                    ;   D5       D4       D3
                    ;PPPPPPPP PPPooSSS SSo-----
    
    bcf     D3, 0
    btfsc   C1, 1
    bsf     D3, 0       
    movlw   b'11000001' ;D3/4/5 are now correct and left aligned
    andlw   D3          ;  including the MSB of C


    rrf     C0, W
    movwf   D2
    bcf     D2, 7   ;Place the MIDDLE 8 bits of C into D2
    btfsc   C1, 0   ;  nb.  C LSB still remians to be done
    bsf     D2, 7

                    ;   D5       D4       D3       D2
                    ;PPPPPPPP PPPooSSS SSoooooC CCCCCCCC   (C lsb missing)

    movf    G0, W          
    movwf   D0              ;ooGGGGGG 
    rlf     D0
    rlf     D1
    rlf     D0
    rlf     D1
    rlf     D0
    rlf     D1
    rlf     D0
    rlf     D1
    rlf     D0
    rlf     D1
                    ;  D1       D0
                    ;-----GGG GGG-----


    movlw   b'00000111' ;Clean them up
    andwf   D1
    movlw   b'11100000'
    andwf   D0
    btfsc   C0,0        ;Finally r ecover the LSB of C
    bsf     D1,7        ;  and place in MSB of D1

                    ; Structure of D should now be be :
                    ;       D5      D4       D3       D2       D1      D0
                    ;   PPPPPPPP PPPooSSS SSoooooC CCCCCCCC CooooGGG GGGooooo
                    ;   P1.\...P0../             \C1\..C0.../        

    btfss   DEBUG
    goto    NoDebug2
    movlw   "C"
    call    SendByte
    movlw   "W"
    call    SendByte
    movlw   "2"
    call    SendByte
    movlw   " "
    call    SendByte
    movlw   d'43'
    call    DebugSendBack
NoDebug2

    movlw   d'43'
    call    ClockItIn

    return
;----------------------------------------------
ClockItIn           ;Clock in the 40 or 43 left-aligned bits in D0/1/2/3/4/5
    movwf   Counter
    bcf     LMXClock
SendNLoop
    rlf     D0
    rlf     D1
    rlf     D2
    rlf     D3
    rlf     D4
    rlf     D5
    btfsc   STATUS, C
    bsf     LMXData
    btfss   STATUS, C
    bcf     LMXData
    bsf     LMXClock
    nop
    bcf     LMXClock
    decfsz  Counter
    goto    SendNLoop

    bsf     LMXStrb
    nop
    bcf     LMXStrb

    return
;------------------------------------
DebugSendBackOld           ;Send D5 - D0  on RS232
    swapf   D5, W
    call    SendHex
    movf    D5, W
    call    SendHex
    movlw   " "
    call    SendByte

    swapf   D4, W
    call    SendHex
    movf    D4, W
    call    SendHex
    movlw   " "
    call    SendByte

    swapf   D3, W
    call    SendHex
    movf    D3, W
    call    SendHex
    movlw   " "
    call    SendByte

    swapf   D2, W
    call    SendHex
    movf    D2, W
    call    SendHex
    movlw   " "
    call    SendByte

    swapf   D1, W
    call    SendHex
    movf    D1, W
    call    SendHex
    movlw   " "
    call    SendByte

    swapf   D0, W
    call    SendHex
    movf    D0, W
    call    SendHex
    movlw   " "
    call    SendByte
    call    CrLf

    return
;------------------------------------
DebugSendBack
    movwf   Counter2

    movf    D5, W
    movwf   Stack6      ;Use Stack as a temporary variable set
    movf    D4, W
    movwf   Stack5
    movf    D3, W
    movwf   Stack4
    movf    D2, W
    movwf   Stack3
    movf    D1, W
    movwf   Stack2
    movf    D0, W
    movwf   Stack1

DBugLoop
    rlf     Stack1    
    rlf     Stack2
    rlf     Stack3
    rlf     Stack4    
    rlf     Stack5    
    rlf     Stack6

    movlw   "0"
    btfsc   STATUS, C
    movlw   "1"
    call    SendByte

    decfsz  Counter2
    goto    DBugLoop

    call    CrLf

    return
;------------------------------------
SendBack                ;Send contents of R and N registers on RS232 interface
    call    CrLf
    movlw   "R"
    call    SendByte
    movlw   " "
    call    SendByte

    swapf   R1, W
    call    SendHex
    movf    R1, W
    call    SendHex
    swapf   R0, W
    call    SendHex
    movf    R0, W
    call    SendHex

    call    CrLf
    movlw   "P"
    call    SendByte
    movlw   " "
    call    SendByte

    movf    P1, W
    call    SendHex
    swapf   P0, W
    call    SendHex
    movf    P0, W
    call    SendHex

    call    CrLf
    movlw   "S"
    call    SendByte
    movlw   " "
    call    SendByte

    swapf   S0, W
    call    SendHex
    movf    S0, W
    call    SendHex

    call    CrLf
    movlw   "C"
    call    SendByte
    movlw   " "
    call    SendByte

    movf    C1, W
    call    SendHex
    swapf   C0, W
    call    SendHex
    movf    C0, W
    call    SendHex

    call    CrLf
    movlw   "G"
    call    SendByte
    movlw   " "
    call    SendByte

    swapf   G0, W
    call    SendHex
    movf    G0, W
    call    SendHex

    call    CrLf

    return
;------------------------------------
DecodeHex           ;Single ASCII character to binary 
    movwf   Temp            ;Uses Temp
    movlw   0x30
    subwf   Temp            ;subtract $30 and leave in Temp
    btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
    clrf    Temp

    movf    Temp , W        
    sublw   d'9'            ;W = 9 - W
    btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
    goto    ConvDone
    movlw   d'7'
    subwf   Temp            ;
ConvDone
    movf    Temp , W        ;
    andlw   0x0F            ;mask out any rubbish left in
    return              
;------------------------------------
SendHex         ;Send a single Nibble as RS232
    andlw   0x0F
    addlw   0x30
    movwf   Temp
    sublw   0x39
    btfsc   STATUS , C
    goto    CharDone
    movlw   7
    addwf   Temp
CharDone
    movf    Temp , W
    call    SendByte
    return
;-----------------------------------
IntroMessage
    movlw   "V"
    call    SendByte
    movlw   "e"
    call    SendByte
    movlw   "r"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "i"
    call    SendByte
    movlw   "c"
    call    SendByte
    movlw   "o"
    call    SendByte
    movlw   "m"
    call    SendByte
    movlw   " "
    call    SendByte

    movlw   "S"
    call    SendByte
    movlw   "y"
    call    SendByte
    movlw   "n"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "h"
    call    SendByte
    movlw   " "
    call    SendByte


    movlw   "C"
    call    SendByte
    movlw   "o"
    call    SendByte
    movlw   "n"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "r"
    call    SendByte
    movlw   "o"
    call    SendByte
    movlw   "l"
    call    SendByte

    call    CrLf


    movlw   "G"
    call    SendByte
    movlw   "4"
    call    SendByte
    movlw   "J"
    call    SendByte
    movlw   "N"
    call    SendByte
    movlw   "T"
    call    SendByte

    movlw   "/"
    call    SendByte
    movlw   "W"
    call    SendByte
    movlw   "W"
    call    SendByte
    movlw   "2"
    call    SendByte
    movlw   "R"
    call    SendByte

    call    CrLf
    call    CrLf

    movlw   "C"
    call    SendByte
    movlw   "o"
    call    SendByte
    movlw   "m"
    call    SendByte
    movlw   "m"
    call    SendByte
    movlw   "a"
    call    SendByte
    movlw   "n"
    call    SendByte
    movlw   "d"
    call    SendByte
    movlw   "s"
    call    SendByte
    movlw   ":"
    call    SendByte
    call    CrLf

    movlw   "R"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    call    CrLf

    movlw   "P"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    call    CrLf

    movlw   "S"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    call    CrLf

    movlw   "C"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    call    CrLf

    movlw   "G"
    call    SendByte
    movlw   "x"
    call    SendByte
    movlw   "x"
    call    SendByte
    call    CrLf

    movlw   "["
    call    SendByte
    movlw   "U"
    call    SendByte
    movlw   "]"
    call    SendByte
    movlw   "p"
    call    SendByte
    movlw   "d"
    call    SendByte
    movlw   "a"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "e"
    call    SendByte
    call    CrLf

    movlw   "["
    call    SendByte
    movlw   "W"
    call    SendByte
    movlw   "]"
    call    SendByte
    movlw   "r"
    call    SendByte
    movlw   "i"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "e"
    call    SendByte
    movlw   " "
    call    SendByte
    movlw   "E"
    call    SendByte
    movlw   "E"
    call    SendByte

    call    CrLf

    return
;-----------------------------------
CrLf
    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte
    return
;---------------------------------
SendByte                    
	movwf   Temp
 if RS232POL == 1
    bsf     SerOut
 else
    bcf     SerOut             ;start bit
 endif
	call    HalfBitDelay        ;
	call    HalfBitDelay
	movlw   8
	movwf   Counter
SendDatLoop
	rrf     Temp                ;bit into C

 if RS232POL == 1
	btfsc   STATUS , C
    bcf     SerOut
	btfss   STATUS , C
    bsf     SerOut             ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
    bsf     SerOut
	btfss   STATUS , C
    bcf     SerOut             ;could have 2us jitter depending on 1/0
 endif

	call    HalfBitDelay
	call    HalfBitDelay
	decfsz  Counter
	goto    SendDatLoop         ;loop 8 + BitDelay long

 if RS232POL == 1
    bcf     SerOut
 else
    bsf     SerOut
 endif
	call    HalfBitDelay    
	call    HalfBitDelay        ;Two stop bits
	call    HalfBitDelay
	call    HalfBitDelay

	return
;-------------------------------
CheckSerCmd             ;[cr] terminator has been received, so
                        ;  decode the commands in here.  Will be in Stack1
    movlw   "R"         ;Rxxxx  R register data data
    subwf   Stack6, W   ;Stack6/5/4/3/2/1
    btfsc   STATUS, Z   ;     R x x x x [cr]
    goto    RCmd  

    movlw   "P"         ;Pxxx   New P register data
    subwf   Stack5, W   ;
    btfsc   STATUS, Z   ;Stack6/5/4/3/2/1
    goto    PCmd        ;       P x x x [cr]

    movlw   "C"         ;Cxxx   New C register data
    subwf   Stack5, W   ;
    btfsc   STATUS, Z   ;Stack6/5/4/3/2/1
    goto    CCmd        ;       C x x x [cr]

    movlw   "S"
    subwf   Stack4, W   ;
    btfsc   STATUS, Z   ;Stack6/5/4/3/2/1
    goto    SCmd        ;         S x x [cr]

    movlw   "G"
    subwf   Stack4, W   ;
    btfsc   STATUS, Z   ;Stack6/5/4/3/2/1
    goto    GCmd        ;         G x x [cr]


    movlw   "U"         ;Update Chip and continue
    subwf   Stack2, W   ;Stack2/1
    btfsc   STATUS, Z   ;Stack7/6/5/4/3/2/1
    goto    UCmd        ;               U [cr]

    movlw   "W"         ;Write to EE and continue
    subwf   Stack2, W   ;Stack2/1
    btfsc   STATUS, Z   ;Stack7/6/5/4/3/2/1
    goto    WCmd        ;               W [cr]

    movlw   "D"         ;Toggle the debugging flag
    subwf   Stack2, W   ;Stack2/1
    btfsc   STATUS, Z   ;Stack7/6/5/4/3/2/1
    goto    DCmd        ;               D [cr]

    goto    DoneCmd
;............
DCmd
    movlw   1
    xorwf   Flags       ;Toggle the debug flag



    movlw   "D"
    call    SendByte
    movlw   "E"
    call    SendByte
    movlw   "B"
    call    SendByte
    movlw   "U"
    call    SendByte
    movlw   "G"
    call    SendByte
    movlw   " "
    call    SendByte

    movlw   "0"
    btfsc   DEBUG
    movlw   "1"
    call    SendByte
    call    CrLf

    goto    DoneCmd
;.............
RCmd
    movf    Stack5 , W      ;  convert to binary in R1/0
    call    DecodeHex       ;MS Nibble
    movwf   R1              ;Stack 6/5/4/3/2/1
    swapf   R1              ;      R x x x x [cr]

    movf    Stack4 , W
    call    DecodeHex
    addwf   R1

    movf    Stack3 , W
    call    DecodeHex
    movwf   R0
    swapf   R0

    movf    Stack2 , W      ;LSNibble
    call    DecodeHex
    addwf   R0

    movlw   0x3F            ;Mask to 14 bits
    andwf   R1

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte

    movlw   "R"
    call    SendByte
    movlw   "-"
    call    SendByte

    swapf   R1 , W          ;Return data on serial link
    call    SendHex
    movf    R1 , W
    call    SendHex
    swapf   R0 , W
    call    SendHex
    movf    R0 , W
    call    SendHex

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte         ;R1/0 updated, but not written to EE

    goto    DoneCmd
;.............
PCmd
    movf    Stack4 , W      ;  convert to binary in C1/0
    call    DecodeHex       ;MS Nibble
    movwf   P1              ;Stack 6/5/4/3/2/1
                            ;        P x x x [cr]
    movf    Stack3 , W
    call    DecodeHex
    movwf   P0
    swapf   P0

    movf    Stack2 , W      ;LSNibble
    call    DecodeHex
    addwf   P0

    movlw   0x07
    andwf   P1              ;mask to 11 bits

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte

    movlw   "P"
    call    SendByte
    movlw   "-"
    call    SendByte

    movf    P1 , W          ;Return data on serial link
    call    SendHex
    swapf   P0 , W
    call    SendHex
    movf    P0 , W
    call    SendHex

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte         ;R1/0 updated, but not written to EE

    goto    DoneCmd
;................
SCmd
    movf    Stack3 , W
    call    DecodeHex
    movwf   S0
    swapf   S0

    movf    Stack2 , W      ;LSNibble
    call    DecodeHex
    addwf   S0

    movlw   0x1F
    andwf   S0              ;Mask to 5 bits

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte

    movlw   "S"
    call    SendByte
    movlw   "-"
    call    SendByte

    swapf   S0 , W
    call    SendHex
    movf    S0 , W
    call    SendHex

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte         ;S0 updated, but not written to EE

    goto    DoneCmd
;..................
GCmd
    movf    Stack3 , W
    call    DecodeHex
    movwf   G0
    swapf   G0

    movf    Stack2 , W      ;LSNibble
    call    DecodeHex
    addwf   G0

    movlw   0x3F
    andwf   G0              ;Mask to 6 bits

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte

    movlw   "G"
    call    SendByte
    movlw   "-"
    call    SendByte

    swapf   G0 , W
    call    SendHex
    movf    G0 , W
    call    SendHex


    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte         ;G updated, but not written to EE

    goto    DoneCmd
;.............
CCmd
    movf    Stack4 , W      ;  convert to binary in R1/0
    call    DecodeHex       ;MS Nibble
    movwf   C1              ;Stack 6/5/4/3/2/1
                            ;        C x x x [cr]
    movf    Stack3 , W
    call    DecodeHex
    movwf   C0
    swapf   C0

    movf    Stack2 , W      ;LSNibble
    call    DecodeHex
    addwf   C0

    movlw   0x03
    andwf   C1              ;Mask to 10 bits

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte

    movlw   "C"
    call    SendByte
    movlw   "-"
    call    SendByte

    movf    C1 , W          ;Return data on serial link
    call    SendHex
    swapf   C0 , W
    call    SendHex
    movf    C0 , W
    call    SendHex

    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte         ;C1/0 updated, but not written to EE

    goto    DoneCmd
;................
UCmd
    call    SendBack
    call    SendData

    movlw   "U"
    call    SendByte
    movlw   "p"
    call    SendByte
    movlw   "d"
    call    SendByte
    movlw   "a"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "e"
    call    SendByte
    movlw   "d"
    call    SendByte
    call    CrLf
    goto    DoneCmd
;..............
WCmd
    call    SendBack        ;Send back contents of actual R and N registers
    call    SendData

    call    WriteEE
    movlw   "W"
    call    SendByte
    movlw   "r"
    call    SendByte
    movlw   "i"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "t"
    call    SendByte
    movlw   "e"
    call    SendByte
    movlw   "n"
    call    SendByte
    call    CrLf
    goto    DoneCmd
;..............
;.............
DoneCmd
    call    ClearStack      ;Prevents spurious commands
    return
;------------------------------- 
Assemble232             ;Get here after Port B interrupt
    call    HalfBitDelay

 if RS232POL == 1
    btfss   SerIn     
 else
    btfsc   Serin
 endif

    goto    GetOut232   ;Invalid start bit

    movlw   d'8'        ;27 clocks after edge to here
    movwf   Counter
    clrf    RxChar
Rx232Loop
    call    HalfBitDelay
    call    HalfBitDelay
    bcf     STATUS , C

 if RS232POL == 1
    btfss   SerIn
 else
    btfsc   SerIn
 endif

    bsf     STATUS , C
    rrf     RxChar            
    decfsz  Counter
    goto    Rx232Loop   ;this loop 2 * HalfBitDelay + 9
    call    HalfBitDelay ;Part into stop bit - should be no more transitions


    movlw   "a"             
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar, W
    call    PushStack       ;RxChar >> Stack1, previous 5 chars move down one
    movf    RxChar , w      ;Returns with data in W as well as on stack
    
GetOut232
    return                  ;received byte in RxChar and W
;-------------------------
HalfBitDelay            ;1200 baud needs 833us, so delay 414 clocks here
    movlw   BAUDCONST   ; for a total of 414 * 2 + 8 = 833 clocks
	movwf   DelCount
BaudDelLoop             ;Total delay = 3.N + 5 = 414
	decfsz  DelCount
	goto    BaudDelLoop
	return
;-------------------------------
LongDelay
    movlw   d'240'      ;Approx 300ms delay
    movwf   Temp2
LongDelLoop2
    movlw   d'249'      
    movwf   DelCount
LongDelLoop1            ;
    nop
    nop
	decfsz  DelCount
    goto    LongDelLoop1

    decfsz  Temp2
    goto    LongDelLoop2
    return
;------------------------------------

GetEE_12F                   ;Enter with address in W
    bsf     STATUS, RP0
    movwf   EEADR
    bsf     EECON1, RD
    movf    EEDATA, W
    bcf     STATUS, RP0
    return                  ;Exit with data in W
;-----------------------------------------
WriteEE
    clrf    AutoEEADR
    movf    R1, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    R0, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    P1, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    P0, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    S0, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    C1, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    C0, W
    call    StoreEE_12F

    incf    AutoEEADR
    movf    G0, W
    call    StoreEE_12F

    return
;-------------------------------------------
StoreEE_12F        ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0      ;All EE addresses in bank 1
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     EECON1 , WREN    ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS , RP0
EEwaitW
    btfss   PIR1 , EEIF
    goto    EEwaitW

    bcf     PIR1 , EEIF      ;has to be cleared manually
    return
;-------------------------
PushStack
    movf    Stack7 , W
    movwf   Stack8              ;Stack8 contains oldest data
    movf    Stack6 , W
    movwf   Stack7
    movf    Stack5 , W
    movwf   Stack6          
    movf    Stack4 , W
    movwf   Stack5
    movf    Stack3 , W
    movwf   Stack4
    movf    Stack2 , W
    movwf   Stack3
    movf    Stack1 , W
    movwf   Stack2
    movf    RxChar , W
    movwf   Stack1
    return
;-------------------------
ClearStack
    movlw   0
    movwf   Stack8
    movwf   Stack7
    movwf   Stack6
    movwf   Stack5
    movwf   Stack4
    movwf   Stack3
    movwf   Stack2
    movwf   Stack1
    return
;------------------------------------
    cblock 0x20         ;64 Bytes available
        Wreg
        StsReg
        Temp
        Temp2
        Counter
        Counter2
        DelCount
        AutoEEADR
        Flags
        D0
        D1
        D2
        D3
        D4
        D5
        R0
        R1
        P0
        P1
        S0
        C0
        C1
        G0
        RxChar
        Stack1
        Stack2
        Stack3
        Stack4
        Stack5
        Stack6
        Stack7
        Stack8
    endc
;----------------------
	end

